@@ -12,16 +12,17 @@ from pathlib import Path
 from typing import Set, Type, List, Tuple, Optional, Iterable, Generator
 
 from colorama import Fore
+from packaging.version import LegacyVersion
 from packaging.version import parse as parse_version
 
 from bauh.api.abstract.context import ApplicationContext
 from bauh.api.abstract.controller import SoftwareManager, SearchResult, UpgradeRequirements, UpgradeRequirement, \
-    TransactionResult, SoftwareAction
+    TransactionResult, SoftwareAction, SettingsView, SettingsController
 from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import SoftwarePackage, PackageHistory, PackageUpdate, PackageSuggestion, \
     SuggestionPriority, CustomSoftwareAction
-from bauh.api.abstract.view import MessageType, ViewComponent, FormComponent, InputOption, SingleSelectComponent, \
+from bauh.api.abstract.view import MessageType, FormComponent, InputOption, SingleSelectComponent, \
     SelectViewType, TextInputComponent, PanelComponent, FileChooserComponent, ViewObserver
 from bauh.api.paths import DESKTOP_ENTRIES_DIR
 from bauh.commons import resource
@@ -66,7 +67,7 @@ class ManualInstallationFileObserver(ViewObserver):
             self.version.set_value(None)
 
 
-class AppImageManager(SoftwareManager):
+class AppImageManager(SoftwareManager, SettingsController):
 
     def __init__(self, context: ApplicationContext):
         super(AppImageManager, self).__init__(context=context)
@@ -79,35 +80,36 @@ class AppImageManager(SoftwareManager):
         self.file_downloader = context.file_downloader
         self.configman = AppImageConfigManager()
         self._custom_actions: Optional[Iterable[CustomSoftwareAction]] = None
-        self.custom_app_actions = (CustomSoftwareAction(i18n_label_key='appimage.custom_action.manual_update',
-                                                        i18n_status_key='appimage.custom_action.manual_update.status',
-                                                        manager_method='update_file',
-                                                        requires_root=False,
-                                                        icon_path=resource.get_path('img/upgrade.svg', ROOT_DIR),
-                                                        requires_confirmation=False),)
+        self._action_self_install: Optional[CustomSoftwareAction] = None
+        self._app_github: Optional[str] = None
+        self._search_unfilled_attrs: Optional[Tuple[str, ...]] = None
 
-    def install_file(self, root_password: str, watcher: ProcessWatcher) -> bool:
+    def install_file(self, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
+        max_width = 350
         file_chooser = FileChooserComponent(label=self.i18n['file'].capitalize(),
                                             allowed_extensions={'AppImage', '*'},
-                                            search_path=get_default_manual_installation_file_dir())
-        input_name = TextInputComponent(label=self.i18n['name'].capitalize())
-        input_version = TextInputComponent(label=self.i18n['version'].capitalize())
+                                            search_path=get_default_manual_installation_file_dir(),
+                                            max_width=max_width)
+        input_name = TextInputComponent(label=self.i18n['name'].capitalize(), max_width=max_width)
+        input_version = TextInputComponent(label=self.i18n['version'].capitalize(), max_width=max_width)
         file_chooser.observers.append(ManualInstallationFileObserver(input_name, input_version))
 
-        input_description = TextInputComponent(label=self.i18n['description'].capitalize())
+        input_description = TextInputComponent(label=self.i18n['description'].capitalize(), max_width=max_width)
 
         cat_ops = [InputOption(label=self.i18n['category.none'].capitalize(), value=0)]
         cat_ops.extend([InputOption(label=self.i18n.get(f'category.{c.lower()}', c.lower()).capitalize(), value=c) for c in self.context.default_categories])
         inp_cat = SingleSelectComponent(label=self.i18n['category'], type_=SelectViewType.COMBO, options=cat_ops,
-                                        default_option=cat_ops[0])
+                                        default_option=cat_ops[0], max_width=max_width)
 
-        form = FormComponent(label='', components=[file_chooser, input_name, input_version, input_description, inp_cat], spaces=False)
+        form = FormComponent(label='', components=[file_chooser, input_name, input_version, input_description, inp_cat],
+                             spaces=False)
 
         while True:
             if watcher.request_confirmation(title=self.i18n['appimage.custom_action.install_file.details'], body=None,
                                             components=[form],
                                             confirmation_label=self.i18n['proceed'].capitalize(),
-                                            deny_label=self.i18n['cancel'].capitalize()):
+                                            deny_label=self.i18n['cancel'].capitalize(),
+                                            min_height=100, max_width=max_width + 150):
                 if not file_chooser.file_path or not os.path.isfile(file_chooser.file_path) or not file_chooser.file_path.lower().strip().endswith('.appimage'):
                     watcher.request_confirmation(title=self.i18n['error'].capitalize(),
                                                  body=self.i18n['appimage.custom_action.install_file.invalid_file'],
@@ -121,7 +123,7 @@ class AppImageManager(SoftwareManager):
             else:
                 return False
 
-        appim = AppImage(i18n=self.i18n, imported=True, custom_actions=self.custom_app_actions)
+        appim = AppImage(i18n=self.i18n, imported=True)
         appim.name = input_name.get_value().strip()
         appim.local_file_path = file_chooser.file_path
         appim.version = input_version.get_value()
@@ -140,18 +142,21 @@ class AppImageManager(SoftwareManager):
 
         return res
 
-    def update_file(self, pkg: AppImage, root_password: str, watcher: ProcessWatcher):
+    def update_file(self, pkg: AppImage, root_password: Optional[str], watcher: ProcessWatcher):
+        max_width = 350
         file_chooser = FileChooserComponent(label=self.i18n['file'].capitalize(),
                                             allowed_extensions={'AppImage', '*'},
-                                            search_path=get_default_manual_installation_file_dir())
-        input_version = TextInputComponent(label=self.i18n['version'].capitalize())
+                                            search_path=get_default_manual_installation_file_dir(),
+                                            max_width=max_width)
+        input_version = TextInputComponent(label=self.i18n['version'].capitalize(), max_width=max_width)
         file_chooser.observers.append(ManualInstallationFileObserver(None, input_version))
 
         while True:
             if watcher.request_confirmation(title=self.i18n['appimage.custom_action.manual_update.details'], body=None,
                                             components=[FormComponent(label='', components=[file_chooser, input_version], spaces=False)],
                                             confirmation_label=self.i18n['proceed'].capitalize(),
-                                            deny_label=self.i18n['cancel'].capitalize()):
+                                            deny_label=self.i18n['cancel'].capitalize(),
+                                            min_height=100, max_width=max_width + 150):
 
                 if not file_chooser.file_path or not os.path.isfile(file_chooser.file_path) or not file_chooser.file_path.lower().strip().endswith('.appimage'):
                     watcher.request_confirmation(title=self.i18n['error'].capitalize(),
@@ -198,7 +203,7 @@ class AppImageManager(SoftwareManager):
 
             idx = 0
             for r in cursor.fetchall():
-                app = AppImage(*r, i18n=self.i18n, custom_actions=self.custom_app_actions)
+                app = AppImage(*r, i18n=self.i18n)
                 not_installed.append(app)
                 found_map[self._gen_app_key(app)] = {'app': app, 'idx': idx}
                 idx += 1
@@ -223,6 +228,11 @@ class AppImageManager(SoftwareManager):
                     new_found = found_map.get(key)
 
                     if new_found:
+                        if not appim.imported:
+                            for attr in self.search_unfilled_attrs:
+                                if getattr(appim, attr) is None:
+                                    setattr(appim, attr, getattr(new_found['app'], attr))
+
                         del not_installed[new_found['idx']]
                         installed_found.append(appim)
                         found = True
@@ -250,7 +260,7 @@ class AppImageManager(SoftwareManager):
                 for path in installed:
                     if path:
                         with open(path) as f:
-                            app = AppImage(installed=True, i18n=self.i18n, custom_actions=self.custom_app_actions, **json.loads(f.read()))
+                            app = AppImage(installed=True, i18n=self.i18n, **json.loads(f.read()))
                             app.icon_url = app.icon_path
 
                         installed_apps.append(app)
@@ -304,7 +314,7 @@ class AppImageManager(SoftwareManager):
         res.total = len(res.installed)
         return res
 
-    def downgrade(self, pkg: AppImage, root_password: str, watcher: ProcessWatcher) -> bool:
+    def downgrade(self, pkg: AppImage, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         versions = self.get_history(pkg)
 
         if len(versions.history) == 1:
@@ -348,7 +358,7 @@ class AppImageManager(SoftwareManager):
 
             return False
 
-    def upgrade(self, requirements: UpgradeRequirements, root_password: str, watcher: ProcessWatcher) -> bool:
+    def upgrade(self, requirements: UpgradeRequirements, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         not_upgraded = []
 
         for req in requirements.to_upgrade:
@@ -387,7 +397,7 @@ class AppImageManager(SoftwareManager):
         watcher.change_substatus('')
         return not all_failed
 
-    def uninstall(self, pkg: AppImage, root_password: str, watcher: ProcessWatcher, disk_loader: DiskCacheLoader = None) -> TransactionResult:
+    def uninstall(self, pkg: AppImage, root_password: Optional[str], watcher: ProcessWatcher, disk_loader: DiskCacheLoader = None) -> TransactionResult:
         if os.path.exists(pkg.get_disk_cache_path()):
             handler = ProcessHandler(watcher)
 
@@ -414,8 +424,22 @@ class AppImageManager(SoftwareManager):
                 if watcher:
                     watcher.print(f"[error] {msg}")
 
+        self._add_self_latest_version(pkg)  # only for self installation
         return TransactionResult(success=True, installed=None, removed=[pkg])
 
+    def _add_self_latest_version(self, app: AppImage):
+        if app.name == self.context.app_name and app.github == self.app_github and not app.url_download_latest_version:
+            history = self.get_history(app)
+
+            if not history or not history.history:
+                self.logger.warning(f"Could not retrieve '{app.name}' versions. "
+                                    f"It will not be possible to determine the current latest version")
+            else:
+                app.version = history.history[0]['0_version']
+                app.latest_version = app.version
+                app.url_download = history.history[0]['2_url_download']
+                app.url_download_latest_version = app.url_download
+
     def get_managed_types(self) -> Set[Type[SoftwarePackage]]:
         return {AppImage}
 
@@ -425,6 +449,11 @@ class AppImageManager(SoftwareManager):
     def get_info(self, pkg: AppImage) -> dict:
         data = pkg.get_data_to_cache()
 
+        if not pkg.installed:
+            for key in ('install_dir', 'symlink', 'icon_path'):
+                if key in data:
+                    del data[key]
+
         if data.get('url_download'):
             size = self.http_client.get_content_length(data['url_download'])
             if size:
@@ -440,6 +469,9 @@ class AppImageManager(SoftwareManager):
 
         return data
 
+    def _sort_release(self, rel: tuple):
+        return rel[0]
+
     def get_history(self, pkg: AppImage) -> PackageHistory:
         history = []
         res = PackageHistory(pkg, history, -1)
@@ -477,12 +509,16 @@ class AppImageManager(SoftwareManager):
             releases = cursor.execute(query.FIND_RELEASES_BY_APP_ID.format(app_tuple[0]))
 
             if releases:
-                for idx, tup in enumerate(releases):
-                    history.append({'0_version': tup[0],
+                treated_releases = [(LegacyVersion(r[0]), *r[1:]) for r in releases]
+                treated_releases.sort(key=self._sort_release, reverse=True)
+
+                for idx, tup in enumerate(treated_releases):
+                    ver = str(tup[0])
+                    history.append({'0_version': ver,
                                     '1_published_at': datetime.strptime(tup[2], '%Y-%m-%dT%H:%M:%SZ') if tup[
                                         2] else '', '2_url_download': tup[1]})
 
-                    if res.pkg_status_idx == -1 and pkg.version == tup[0]:
+                    if res.pkg_status_idx == -1 and pkg.version == ver:
                         res.pkg_status_idx = idx
 
                 return res
@@ -536,10 +572,11 @@ class AppImageManager(SoftwareManager):
 
         return file_name, file_path
 
-    def install(self, pkg: AppImage, root_password: str, disk_loader: Optional[DiskCacheLoader], watcher: ProcessWatcher) -> TransactionResult:
+    def install(self, pkg: AppImage, root_password: Optional[str], disk_loader: Optional[DiskCacheLoader], watcher: ProcessWatcher) -> TransactionResult:
         return self._install(pkg=pkg, watcher=watcher)
 
-    def _install(self, pkg: AppImage, watcher: ProcessWatcher, pre_downloaded_file: Optional[Tuple[str, str]] = None):
+    def _install(self, pkg: AppImage, watcher: ProcessWatcher, pre_downloaded_file: Optional[Tuple[str, str]] = None) \
+            -> TransactionResult:
 
         handler = ProcessHandler(watcher)
         out_dir = f'{INSTALLATION_DIR}/{pkg.get_clean_name()}'
@@ -696,7 +733,7 @@ class AppImageManager(SoftwareManager):
     def requires_root(self, action: SoftwareAction, pkg: AppImage) -> bool:
         return False
 
-    def prepare(self, task_manager: TaskManager, root_password: str, internet_available: bool):
+    def prepare(self, task_manager: TaskManager, root_password: Optional[str], internet_available: bool):
         create_config = CreateConfigFile(taskman=task_manager, configman=self.configman, i18n=self.i18n,
                                          task_icon_path=get_icon_path(), logger=self.logger)
         create_config.start()
@@ -730,7 +767,7 @@ class AppImageManager(SoftwareManager):
         if not dbfiles or len({f for f in (DATABASE_APPS_FILE, DATABASE_RELEASES_FILE) if f in dbfiles}) != 2:
             return [self.i18n['appimage.warning.missing_db_files'].format(appimage=bold('AppImage'))]
 
-    def list_suggestions(self, limit: int, filter_installed: bool) -> List[PackageSuggestion]:
+    def list_suggestions(self, limit: int, filter_installed: bool) -> Optional[List[PackageSuggestion]]:
         res = []
 
         connection = self._get_db_connection(DATABASE_APPS_FILE)
@@ -768,7 +805,7 @@ class AppImageManager(SoftwareManager):
                     cursor.execute(query.FIND_APPS_BY_NAME_FULL.format(','.join([f"'{s}'" for s in sugs_map.keys()])))
 
                     for t in cursor.fetchall():
-                        app = AppImage(*t, i18n=self.i18n, custom_actions=self.custom_app_actions)
+                        app = AppImage(*t, i18n=self.i18n)
                         res.append(PackageSuggestion(app, sugs_map[app.name.lower()]))
                     self.logger.info(f"Mapped {len(res)} suggestions")
                 except:
@@ -795,11 +832,9 @@ class AppImageManager(SoftwareManager):
     def cache_to_disk(self, pkg: SoftwarePackage, icon_bytes: Optional[bytes], only_icon: bool):
         self.serialize_to_disk(pkg, icon_bytes, only_icon)
 
-    def get_screenshots(self, pkg: AppImage) -> List[str]:
-        if pkg.has_screenshots():
-            return [pkg.url_screenshot]
-
-        return []
+    def get_screenshots(self, pkg: AppImage) -> Generator[str, None, None]:
+        if pkg.url_screenshot:
+            yield pkg.url_screenshot
 
     def clear_data(self, logs: bool = True):
         for f in glob.glob(f'{APPIMAGE_SHARED_DIR}/*.db'):
@@ -815,38 +850,38 @@ class AppImageManager(SoftwareManager):
                     print(f'{Fore.RED}[bauh][appimage] An exception has happened when deleting {f}{Fore.RESET}')
                     traceback.print_exc()
 
-    def get_settings(self, screen_width: int, screen_height: int) -> Optional[ViewComponent]:
-        appimage_config = self.configman.get_config()
-        max_width = floor(screen_width * 0.15)
+    def get_settings(self) -> Optional[Generator[SettingsView, None, None]]:
+        config_ = self.configman.get_config()
+        max_width = 50
 
         comps = [
             TextInputComponent(label=self.i18n['appimage.config.database.expiration'],
-                               value=int(appimage_config['database']['expiration']) if isinstance(
-                                   appimage_config['database']['expiration'], int) else '',
+                               value=int(config_['database']['expiration']) if isinstance(
+                                   config_['database']['expiration'], int) else '',
                                tooltip=self.i18n['appimage.config.database.expiration.tip'],
                                only_int=True,
                                max_width=max_width,
                                id_='appim_db_exp'),
             TextInputComponent(label=self.i18n['appimage.config.suggestions.expiration'],
-                               value=int(appimage_config['suggestions']['expiration']) if isinstance(
-                                   appimage_config['suggestions']['expiration'], int) else '',
+                               value=int(config_['suggestions']['expiration']) if isinstance(
+                                   config_['suggestions']['expiration'], int) else '',
                                tooltip=self.i18n['appimage.config.suggestions.expiration.tip'],
                                only_int=True,
                                max_width=max_width,
                                id_='appim_sugs_exp')
         ]
 
-        return PanelComponent([FormComponent(components=comps, id_='form')])
+        yield SettingsView(self, PanelComponent([FormComponent(components=comps)]))
 
     def save_settings(self, component: PanelComponent) -> Tuple[bool, Optional[List[str]]]:
-        appimage_config = self.configman.get_config()
+        config_ = self.configman.get_config()
 
-        form = component.get_form_component('form')
-        appimage_config['database']['expiration'] = form.get_text_input('appim_db_exp').get_int_value()
-        appimage_config['suggestions']['expiration'] = form.get_text_input('appim_sugs_exp').get_int_value()
+        form = component.get_component_by_idx(0, FormComponent)
+        config_['database']['expiration'] = form.get_component('appim_db_exp', TextInputComponent).get_int_value()
+        config_['suggestions']['expiration'] = form.get_component('appim_sugs_exp', TextInputComponent).get_int_value()
 
         try:
-            self.configman.save_config(appimage_config)
+            self.configman.save_config(config_)
             return True, None
         except:
             return False, [traceback.format_exc()]
@@ -855,6 +890,7 @@ class AppImageManager(SoftwareManager):
         if self._custom_actions is None:
             self._custom_actions = (CustomSoftwareAction(i18n_label_key='appimage.custom_action.install_file',
                                                          i18n_status_key='appimage.custom_action.install_file.status',
+                                                         i18n_description_key='appimage.custom_action.install_file.desc',
                                                          manager=self,
                                                          manager_method='install_file',
                                                          icon_path=resource.get_path('img/appimage.svg', ROOT_DIR),
@@ -862,14 +898,19 @@ class AppImageManager(SoftwareManager):
                                                          requires_confirmation=False),
                                     CustomSoftwareAction(i18n_label_key='appimage.custom_action.update_db',
                                                          i18n_status_key='appimage.custom_action.update_db.status',
+                                                         i18n_description_key='appimage.custom_action.update_db.desc',
                                                          manager=self,
                                                          manager_method='update_database',
                                                          icon_path=resource.get_path('img/appimage.svg', ROOT_DIR),
                                                          requires_root=False,
                                                          requires_internet=True))
+
+        if self._get_self_appimage_running() and not self._is_self_installed():
+            yield self.action_self_install
+
         yield from self._custom_actions
 
-    def get_upgrade_requirements(self, pkgs: List[AppImage], root_password: str, watcher: ProcessWatcher) -> UpgradeRequirements:
+    def get_upgrade_requirements(self, pkgs: List[AppImage], root_password: Optional[str], watcher: ProcessWatcher) -> UpgradeRequirements:
         to_update = []
 
         for pkg in pkgs:
@@ -931,9 +972,114 @@ class AppImageManager(SoftwareManager):
 
         pkg.updates_ignored = False
 
-    def update_database(self, root_password: str, watcher: ProcessWatcher) -> bool:
+    def update_database(self, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         db_updater = DatabaseUpdater(i18n=self.i18n, http_client=self.context.http_client,
                                      logger=self.context.logger, watcher=watcher, taskman=TaskManager())
 
         res = db_updater.download_databases()
         return res
+
+    @property
+    def search_unfilled_attrs(self) -> Tuple[str, ...]:
+        if self._search_unfilled_attrs is None:
+            self._search_unfilled_attrs = ('icon_url', 'url_download_latest_version', 'author', 'license', 'github',
+                                           'source', 'url_screenshot')
+
+        return self._search_unfilled_attrs
+
+    def self_install(self, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
+        file_path = self._get_self_appimage_running()
+
+        if not file_path:
+            return False
+
+        if self._is_self_installed():
+            return False
+
+        app = AppImage(name=self.context.app_name, version=self.context.app_version,
+                       categories=['system'], author=self.context.app_name, github=self.app_github,
+                       license='zlib/libpng')
+
+        res = self._install(pkg=app, watcher=watcher,
+                            pre_downloaded_file=(os.path.basename(file_path), file_path))
+        if res.success:
+            app.installed = True
+
+            de_path = self._gen_desktop_entry_path(app)
+
+            if de_path and os.path.exists(de_path):
+                with open(de_path) as f:
+                    bauh_entry = f.read()
+
+                if bauh_entry:
+                    comments = re.compile(r'Comment(\[\w+])?\s*=\s*(.+)').findall(bauh_entry)
+
+                    if comments:
+                        locale = f'{self.i18n.current_key}' if self.i18n.current_key != self.i18n.default_key else None
+
+                        for key, desc in comments:
+                            if desc:
+                                if not key:
+                                    app.description = desc  # default description
+
+                                    if not locale:
+                                        break
+
+                                elif key == locale:
+                                    app.description = desc  # localized description
+                                    break
+                    else:
+                        self.context.logger.warning(f"Could not find the 'Comment' fields from {self.context.app_name}'s desktop entry")
+                else:
+                    self.context.logger.warning(f"{self.context.app_name} desktop entry is empty. Is is not possible to determine the 'description' field")
+
+            else:
+                self.context.logger.warning(f"{self.context.app_name} desktop file not found ({de_path}). It is not possible to determine the 'description' field")
+
+            self.cache_to_disk(app, None, False)
+
+        return res.success
+
+    def _is_self_installed(self) -> bool:
+        return os.path.exists(f'{INSTALLATION_DIR}/{self.context.app_name}/data.json')
+
+    def _get_self_appimage_running(self) -> Optional[str]:
+        file = os.getenv('APPIMAGE')
+
+        if not file:
+            return
+
+        app_exec = os.getenv('APPRUN_STARTUP_EXEC_ARGS')
+
+        if not app_exec:
+            return
+
+        if os.path.basename(app_exec).split(' ')[0] != self.context.app_name:
+            return
+
+        if not os.path.exists(file):
+            return
+
+        return file
+
+    @property
+    def action_self_install(self) -> CustomSoftwareAction:
+        if self._action_self_install is None:
+            self._action_self_install = CustomSoftwareAction(i18n_label_key='appimage.custom_action.self_install',
+                                                             i18n_status_key='appimage.custom_action.self_install.status',
+                                                             i18n_description_key='appimage.custom_action.self_install.desc',
+                                                             manager=self,
+                                                             manager_method='self_install',
+                                                             icon_path=resource.get_path('img/appimage.svg', ROOT_DIR),
+                                                             requires_root=False,
+                                                             refresh=True,
+                                                             requires_internet=False)
+
+        return self._action_self_install
+
+    @property
+    def app_github(self) -> str:
+        if self._app_github is None:
+            self._app_github = f'vinifmor/{self.context.app_name}'
+
+        return self._app_github
