@@ -55,10 +55,6 @@ def get_repositories(pkgs: Iterable[str]) -> dict:
     return repositories
 
 
-def is_available_in_repositories(pkg_name: str) -> bool:
-    return bool(run_cmd('pacman -Ss ' + pkg_name))
-
-
 def get_info(pkg_name, remote: bool = False) -> str:
     return run_cmd('pacman -{}i {}'.format('Q' if not remote else 'S', pkg_name), print_error=False)
 
@@ -99,7 +95,7 @@ def _fill_ignored(res: dict):
     res['pkgs'] = list_ignored_packages()
 
 
-def map_installed(names: Iterable[str] = None) -> dict:  # returns a dict with with package names as keys and versions as values
+def map_installed(names: Optional[Iterable[str]] = None) -> Dict[str, Dict[str, str]]:
     ignored = {}
     thread_ignored = Thread(target=_fill_ignored, args=(ignored,), daemon=True)
     thread_ignored.start()
@@ -146,12 +142,14 @@ def map_installed(names: Iterable[str] = None) -> dict:  # returns a dict with w
     return pkgs
 
 
-def install_as_process(pkgpaths: Iterable[str], root_password: str, file: bool, pkgdir: str = '.',
+def install_as_process(pkgpaths: Iterable[str], root_password: Optional[str], file: bool, pkgdir: str = '.',
                        overwrite_conflicting_files: bool = False, simulate: bool = False, as_deps: bool = False) -> SimpleProcess:
     cmd = ['pacman', '-U'] if file else ['pacman', '-S']
     cmd.extend(pkgpaths)
 
-    if not simulate:
+    if simulate:
+        cmd.append('--confirm')
+    else:
         cmd.append('--noconfirm')
         cmd.append('-dd')
 
@@ -165,7 +163,7 @@ def install_as_process(pkgpaths: Iterable[str], root_password: str, file: bool,
                          root_password=root_password,
                          cwd=pkgdir,
                          error_phrases={"error: failed to prepare transaction", 'error: failed to commit transaction', 'error: target not found'},
-                         shell=True)
+                         shell=False)
 
 
 def map_desktop_files(*pkgnames) -> Dict[str, List[str]]:
@@ -211,11 +209,11 @@ def verify_pgp_key(key: str) -> bool:
     return False
 
 
-def receive_key(key: str, root_password: str) -> SystemProcess:
+def receive_key(key: str, root_password: Optional[str]) -> SystemProcess:
     return SystemProcess(new_root_subprocess(['pacman-key', '-r', key], root_password=root_password), check_error_output=False)
 
 
-def sign_key(key: str, root_password: str) -> SystemProcess:
+def sign_key(key: str, root_password: Optional[str]) -> SystemProcess:
     return SystemProcess(new_root_subprocess(['pacman-key', '--lsign-key', key], root_password=root_password), check_error_output=False)
 
 
@@ -361,7 +359,7 @@ def read_dependencies(name: str) -> Set[str]:
     return depends_on
 
 
-def sync_databases(root_password: str, force: bool = False) -> SimpleProcess:
+def sync_databases(root_password: Optional[str], force: bool = False) -> SimpleProcess:
     return SimpleProcess(cmd=['pacman', '-Sy{}'.format('y' if force else '')],
                          root_password=root_password)
 
@@ -395,32 +393,6 @@ def list_repository_updates() -> Dict[str, str]:
     return res
 
 
-def map_sorting_data(pkgnames: List[str]) -> Dict[str, dict]:
-    allinfo = new_subprocess(['pacman', '-Qi', *pkgnames]).stdout
-
-    pkgs, current_pkg = {}, {}
-    mapped_attrs = 0
-    for out in new_subprocess(["grep", "-Po", "(Name|Provides|Depends On)\s*:\s*\K(.+)"], stdin=allinfo).stdout:
-        if out:
-            line = out.decode().strip()
-
-            if line:
-                if mapped_attrs == 0:
-                    current_pkg['name'] = line
-                elif mapped_attrs == 1:
-                    provides = set() if line == 'None' else set(line.split(' '))
-                    provides.add(current_pkg['name'])
-                    current_pkg['provides'] = provides
-                elif mapped_attrs == 2:
-                    current_pkg['depends'] = line.split(':')[1].strip()
-                    pkgs[current_pkg['name']] = current_pkg
-                    del current_pkg['name']
-
-                    mapped_attrs = 0
-                    current_pkg = {}
-    return pkgs
-
-
 def get_build_date(pkgname: str) -> str:
     output = run_cmd('pacman -Qi {}'.format(pkgname))
 
@@ -469,15 +441,15 @@ def can_refresh_mirrors() -> bool:
     return is_mirrors_available()
 
 
-def refresh_mirrors(root_password: str) -> SimpleProcess:
+def refresh_mirrors(root_password: Optional[str]) -> SimpleProcess:
     return SimpleProcess(cmd=['pacman-mirrors', '-g'], root_password=root_password)
 
 
-def update_mirrors(root_password: str, countries: List[str]) -> SimpleProcess:
+def update_mirrors(root_password: Optional[str], countries: List[str]) -> SimpleProcess:
     return SimpleProcess(cmd=['pacman-mirrors', '-c', ','.join(countries)], root_password=root_password)
 
 
-def sort_fastest_mirrors(root_password: str, limit: int) -> SimpleProcess:
+def sort_fastest_mirrors(root_password: Optional[str], limit: int) -> SimpleProcess:
     cmd = ['pacman-mirrors', '--fasttrack']
 
     if limit > 0:
@@ -502,34 +474,34 @@ def is_mirrors_available() -> bool:
     return bool(shutil.which('pacman-mirrors'))
 
 
-def map_update_sizes(pkgs: List[str]) -> Dict[str, int]:  # bytes:
+def map_update_sizes(pkgs: List[str]) -> Dict[str, float]:  # bytes:
     output = run_cmd('pacman -Si {}'.format(' '.join(pkgs)))
 
     if output:
-        return {pkgs[idx]: size_to_byte(float(size[0]), size[1]) for idx, size in enumerate(RE_INSTALLED_SIZE.findall(output))}
+        return {pkgs[idx]: size_to_byte(size[0], size[1]) for idx, size in enumerate(RE_INSTALLED_SIZE.findall(output))}
 
     return {}
 
 
-def map_download_sizes(pkgs: List[str]) -> Dict[str, int]:  # bytes:
+def map_download_sizes(pkgs: List[str]) -> Dict[str, float]:  # bytes:
     output = run_cmd('pacman -Si {}'.format(' '.join(pkgs)))
 
     if output:
-        return {pkgs[idx]: size_to_byte(float(size[0]), size[1]) for idx, size in enumerate(RE_DOWNLOAD_SIZE.findall(output))}
+        return {pkgs[idx]: size_to_byte(size[0], size[1]) for idx, size in enumerate(RE_DOWNLOAD_SIZE.findall(output))}
 
     return {}
 
 
-def get_installed_size(pkgs: List[str]) -> Dict[str, int]:  # bytes
+def get_installed_size(pkgs: List[str]) -> Dict[str, float]:  # bytes
     output = run_cmd('pacman -Qi {}'.format(' '.join(pkgs)))
 
     if output:
-        return {pkgs[idx]: size_to_byte(float(size[0]), size[1]) for idx, size in enumerate(RE_INSTALLED_SIZE.findall(output))}
+        return {pkgs[idx]: size_to_byte(size[0], size[1]) for idx, size in enumerate(RE_INSTALLED_SIZE.findall(output))}
 
     return {}
 
 
-def upgrade_system(root_password: str) -> SimpleProcess:
+def upgrade_system(root_password: Optional[str]) -> SimpleProcess:
     return SimpleProcess(cmd=['pacman', '-Syyu', '--noconfirm'], root_password=root_password)
 
 
@@ -627,40 +599,87 @@ def list_download_data(pkgs: Iterable[str]) -> List[Dict[str, str]]:
     return res
 
 
-def map_updates_data(pkgs: Iterable[str], files: bool = False) -> dict:
-    if files:
-        output = run_cmd('pacman -Qi -p {}'.format(' '.join(pkgs)))
-    else:
-        output = run_cmd('pacman -Si {}'.format(' '.join(pkgs)))
+def map_updates_data(pkgs: Iterable[str], files: bool = False, description: bool = False) -> Optional[Dict[str, Dict[str, object]]]:
+    if pkgs:
+        if files:
+            output = run_cmd('pacman -Qi -p {}'.format(' '.join(pkgs)))
+        else:
+            output = run_cmd('pacman -Si {}'.format(' '.join(pkgs)))
 
-    res = {}
-    if output:
-        latest_name = None
-        data = {'ds': None, 's': None, 'v': None, 'c': None, 'p': None, 'd': None, 'r': None}
-        latest_field = None
+        res = {}
+        if output:
+            latest_name = None
+            data = {'ds': None, 's': None, 'v': None, 'c': None, 'p': None, 'd': None, 'r': None, 'des': None}
+            latest_field = None
 
-        for l in output.split('\n'):
-            if l:
-                if l[0] != ' ':
-                    line = l.strip()
-                    field_sep_idx = line.index(':')
-                    field = line[0:field_sep_idx].strip()
-                    val = line[field_sep_idx + 1:].strip()
+            for l in output.split('\n'):
+                if l:
+                    if l[0] != ' ':
+                        line = l.strip()
+                        field_sep_idx = line.index(':')
+                        field = line[0:field_sep_idx].strip()
+                        val = line[field_sep_idx + 1:].strip()
 
-                    if field == 'Repository':
-                        data['r'] = val
-                        latest_field = 'r'
-                    elif field == 'Name':
-                        latest_name = val
-                        latest_field = 'n'
-                    elif field == 'Version':
-                        data['v'] = val.split('=')[0]
-                        latest_field = 'v'
-                    elif field == 'Provides':
-                        latest_field = 'p'
-                        data['p'] = {latest_name, '{}={}'.format(latest_name, data['v'])}
-                        if val != 'None':
-                            for w in val.split(' '):
+                        if field == 'Repository':
+                            data['r'] = val
+                            latest_field = 'r'
+                        elif field == 'Name':
+                            latest_name = val
+                            latest_field = 'n'
+                        elif field == 'Version':
+                            data['v'] = val.split('=')[0]
+                            latest_field = 'v'
+                        elif description and field == 'Description':
+                            data['des'] = val
+                            latest_field = 'des'
+                        elif field == 'Provides':
+                            latest_field = 'p'
+                            data['p'] = {latest_name, '{}={}'.format(latest_name, data['v'])}
+                            if val != 'None':
+                                for w in val.split(' '):
+                                    if w:
+                                        word = w.strip()
+                                        data['p'].add(word)
+
+                                        word_split = word.split('=')
+
+                                        if word_split[0] != word:
+                                            data['p'].add(word_split[0])
+                        elif field == 'Depends On':
+                            val = val.strip()
+
+                            if val == 'None':
+                                data['d'] = None
+                            else:
+                                data['d'] = {w.strip() for w in val.split(' ') if w}
+                                latest_field = 'd'
+                        elif field == 'Conflicts With':
+                            if val == 'None':
+                                data['c'] = None
+                            else:
+                                data['c'] = {w.strip() for w in val.split(' ') if w}
+
+                            latest_field = 'c'
+                        elif field == 'Download Size':
+                            size = val.split(' ')
+                            data['ds'] = size_to_byte(size[0], size[1])
+                            latest_field = 'ds'
+                        elif field == 'Installed Size':
+                            size = val.split(' ')
+                            data['s'] = size_to_byte(size[0], size[1])
+                            latest_field = 's'
+                        elif latest_name and latest_field == 's':
+                            res[latest_name] = data
+                            latest_name = None
+                            latest_field = None
+                            data = {'ds': None, 's': None, 'c': None, 'p': None, 'd': None,
+                                    'r': None,  'v': None, 'des': None}
+                        else:
+                            latest_field = None
+
+                    elif latest_field and latest_field in ('p', 'c', 'd'):
+                        if latest_field == 'p':
+                            for w in l.split(' '):
                                 if w:
                                     word = w.strip()
                                     data['p'].add(word)
@@ -669,55 +688,13 @@ def map_updates_data(pkgs: Iterable[str], files: bool = False) -> dict:
 
                                     if word_split[0] != word:
                                         data['p'].add(word_split[0])
-                    elif field == 'Depends On':
-                        val = val.strip()
-
-                        if val == 'None':
-                            data['d'] = None
                         else:
-                            data['d'] = {w.strip() for w in val.split(' ') if w}
-                            latest_field = 'd'
-                    elif field == 'Conflicts With':
-                        if val == 'None':
-                            data['c'] = None
-                        else:
-                            data['c'] = {w.strip() for w in val.split(' ') if w}
+                            data[latest_field].update((w.strip() for w in l.split(' ') if w))
 
-                        latest_field = 'c'
-                    elif field == 'Download Size':
-                        size = val.split(' ')
-                        data['ds'] = size_to_byte(float(size[0]), size[1])
-                        latest_field = 'ds'
-                    elif field == 'Installed Size':
-                        size = val.split(' ')
-                        data['s'] = size_to_byte(float(size[0]), size[1])
-                        latest_field = 's'
-                    elif latest_name and latest_field == 's':
-                        res[latest_name] = data
-                        latest_name = None
-                        latest_field = None
-                        data = {'ds': None, 's': None, 'c': None, 'p': None, 'd': None, 'r': None,  'v': None}
-                    else:
-                        latest_field = None
-
-                elif latest_field and latest_field in ('p', 'c', 'd'):
-                    if latest_field == 'p':
-                        for w in l.split(' '):
-                            if w:
-                                word = w.strip()
-                                data['p'].add(word)
-
-                                word_split = word.split('=')
-
-                                if word_split[0] != word:
-                                    data['p'].add(word_split[0])
-                    else:
-                        data[latest_field].update((w.strip() for w in l.split(' ') if w))
-
-    return res
+        return res
 
 
-def upgrade_several(pkgnames: Iterable[str], root_password: str, overwrite_conflicting_files: bool = False, skip_dependency_checks: bool = False) -> SimpleProcess:
+def upgrade_several(pkgnames: Iterable[str], root_password: Optional[str], overwrite_conflicting_files: bool = False, skip_dependency_checks: bool = False) -> SimpleProcess:
     cmd = ['pacman', '-S', *pkgnames, '--noconfirm']
 
     if overwrite_conflicting_files:
@@ -732,14 +709,14 @@ def upgrade_several(pkgnames: Iterable[str], root_password: str, overwrite_confl
                          shell=True)
 
 
-def download(root_password: str, *pkgnames: str) -> SimpleProcess:
+def download(root_password: Optional[str], *pkgnames: str) -> SimpleProcess:
     return SimpleProcess(cmd=['pacman', '-Swdd', *pkgnames, '--noconfirm', '--noprogressbar'],
                          root_password=root_password,
                          error_phrases={'error: failed to prepare transaction', 'error: failed to commit transaction', 'error: target not found'},
                          shell=True)
 
 
-def remove_several(pkgnames: Iterable[str], root_password: str, skip_checks: bool = False) -> SimpleProcess:
+def remove_several(pkgnames: Iterable[str], root_password: Optional[str], skip_checks: bool = False) -> SimpleProcess:
     cmd = ['pacman', '-R', *pkgnames, '--noconfirm']
 
     if skip_checks:
@@ -748,16 +725,26 @@ def remove_several(pkgnames: Iterable[str], root_password: str, skip_checks: boo
     return SimpleProcess(cmd=cmd, root_password=root_password, wrong_error_phrases={'warning:'}, shell=True)
 
 
+def _map_optional_dep(line: str, not_installed: bool) -> Optional[Tuple[str, Optional[str]]]:
+    if not not_installed or not line.endswith('[installed]'):
+        pkg_desc = line.split(':')
+
+        if len(pkg_desc) == 1:
+            return pkg_desc[0].split('[installed]')[0].strip(), ''
+        elif len(pkg_desc) > 1:
+            return pkg_desc[0], pkg_desc[1].split('[installed]')[0].strip()
+
+
 def map_optional_deps(names: Iterable[str], remote: bool, not_installed: bool = False) -> Dict[str, Dict[str, str]]:
     output = run_cmd('pacman -{}i {}'.format('S' if remote else 'Q', ' '.join(names)))
     res = {}
     if output:
         latest_name, deps = None, None
 
-        for l in output.split('\n'):
-            if l:
-                if l[0] != ' ':
-                    line = l.strip()
+        for raw_line in output.split('\n'):
+            if raw_line:
+                if raw_line[0] != ' ':
+                    line = raw_line.strip()
                     field_sep_idx = line.index(':')
                     field = line[0:field_sep_idx].strip()
 
@@ -768,98 +755,24 @@ def map_optional_deps(names: Iterable[str], remote: bool, not_installed: bool =
                         val = line[field_sep_idx + 1:].strip()
                         deps = {}
                         if val != 'None':
-                            if ':' in val:
-                                dep_info = val.split(':')
-                                desc = dep_info[1].strip()
+                            dep_desc = _map_optional_dep(val, not_installed)
 
-                                if desc and not_installed and '[installed]' in desc:
-                                    continue
+                            if dep_desc:
+                                deps[dep_desc[0]] = dep_desc[1]
 
-                                deps[dep_info[0].strip()] = desc
-                            else:
-                                sev_deps = {dep.strip(): '' for dep in val.split(' ') if dep and (not not_installed or '[installed]' not in dep)}
-                                deps.update(sev_deps)
                     elif latest_name and deps is not None:
                         res[latest_name] = deps
                         latest_name, deps = None, None
 
                 elif latest_name and deps is not None:
-                    if ':' in l:
-                        dep_info = l.split(':')
-                        desc = dep_info[1].strip()
+                    dep_desc = _map_optional_dep(raw_line.strip(), not_installed)
 
-                        if desc and not_installed and '[installed]' in desc:
-                            continue
-
-                        deps[dep_info[0].strip()] = desc
-                    else:
-                        sev_deps = {dep.strip(): '' for dep in l.split(' ') if dep and (not not_installed or '[installed]' not in dep)}
-                        deps.update(sev_deps)
+                    if dep_desc:
+                        deps[dep_desc[0]] = dep_desc[1]
 
     return res
 
 
-def map_all_deps(names: Iterable[str], only_installed: bool = False) -> Dict[str, Set[str]]:
-    output = run_cmd('pacman -Qi {}'.format(' '.join(names)))
-
-    if output:
-        res = {}
-        deps_fields = {'Depends On', 'Optional Deps'}
-        latest_name, deps, latest_field = None, None, None
-
-        for l in output.split('\n'):
-            if l:
-                if l[0] != ' ':
-                    line = l.strip()
-                    field_sep_idx = line.index(':')
-                    field = line[0:field_sep_idx].strip()
-
-                    if field == 'Name':
-                        latest_field = field
-                        val = line[field_sep_idx + 1:].strip()
-                        latest_name = val
-                        deps = None
-                    elif field in deps_fields:
-                        latest_field = field
-                        val = line[field_sep_idx + 1:].strip()
-                        opt_deps = latest_field == 'Optional Deps'
-
-                        if deps is None:
-                            deps = set()
-
-                        if val != 'None':
-                            if ':' in val:
-                                dep_info = val.split(':')
-                                desc = dep_info[1].strip()
-
-                                if desc and opt_deps and only_installed and '[installed]' not in desc:
-                                    continue
-
-                                deps.add(dep_info[0].strip())
-                            else:
-                                deps.update({dep.strip() for dep in val.split(' ') if dep})
-
-                    elif latest_name and deps is not None:
-                        res[latest_name] = deps
-                        latest_name, deps, latest_field = None, None, None
-
-                elif latest_name and deps is not None:
-                    opt_deps = latest_field == 'Optional Deps'
-
-                    if ':' in l:
-                        dep_info = l.split(':')
-                        desc = dep_info[1].strip()
-
-                        if desc and opt_deps and only_installed and '[installed]' not in desc:
-                            continue
-
-                        deps.add(dep_info[0].strip())
-                    else:
-                        deps.update({dep.strip() for dep in l.split(' ') if dep})
-
-        return res
-
-
 def map_required_dependencies(*names: str) -> Dict[str, Set[str]]:
     output = run_cmd('pacman -Qi {}'.format(' '.join(names) if names else ''))
 
@@ -1033,114 +946,6 @@ def map_replaces(names: Iterable[str], remote: bool = False) -> Dict[str, Set[st
         return res
 
 
-def _list_unnecessary_deps(pkgs: Iterable[str], already_checked: Set[str], all_provided: Dict[str, Set[str]], recursive: bool = False) -> Set[str]:
-    output = run_cmd('pacman -Qi {}'.format(' '.join(pkgs)))
-
-    if output:
-        res = set()
-        deps_field = False
-
-        for l in output.split('\n'):
-            if l:
-                if l[0] != ' ':
-                    line = l.strip()
-                    field_sep_idx = line.index(':')
-                    field = line[0:field_sep_idx].strip()
-
-                    if field == 'Depends On':
-                        deps_field = True
-                        val = line[field_sep_idx + 1:].strip()
-
-                        if val != 'None':
-                            if ':' in val:
-                                dep_info = val.split(':')
-
-                                real_deps = all_provided.get(dep_info[0].strip())
-
-                                if real_deps:
-                                    res.update(real_deps)
-                            else:
-                                for dep in val.split(' '):
-                                    if dep:
-                                        real_deps = all_provided.get(dep.strip())
-
-                                        if real_deps:
-                                            res.update(real_deps)
-
-                    elif deps_field:
-                        latest_field = False
-
-                elif deps_field:
-                    if ':' in l:
-                        dep_info = l.split(':')
-
-                        real_deps = all_provided.get(dep_info[0].strip())
-
-                        if real_deps:
-                            res.update(real_deps)
-                    else:
-                        for dep in l.split(' '):
-                            if dep:
-                                real_deps = all_provided.get(dep.strip())
-
-                                if real_deps:
-                                    res.update(real_deps)
-
-        if res:
-            res = {dep for dep in res if dep not in already_checked}
-            already_checked.update(res)
-
-            if recursive and res:
-                subdeps = _list_unnecessary_deps(res, already_checked, all_provided)
-
-                if subdeps:
-                    res.update(subdeps)
-
-        return res
-
-
-def list_unnecessary_deps(pkgs: Iterable[str], all_provided: Dict[str, Set[str]] = None) -> Set[str]:
-    all_checked = set(pkgs)
-    all_deps = _list_unnecessary_deps(pkgs, all_checked, map_provided(remote=False) if not all_provided else all_provided, recursive=True)
-
-    unnecessary = set(pkgs)
-    if all_deps:
-        requirements_map = map_required_by(all_deps)
-
-        to_clean = set()
-        for dep, required_by in requirements_map.items():
-            if not required_by or not required_by.difference(unnecessary):
-                unnecessary.add(dep)
-                to_clean.add(dep)
-            elif required_by.difference(all_checked):  # checking if there are requirements outside the context
-                to_clean.add(dep)
-
-        if to_clean:
-            for dep in to_clean:
-                del requirements_map[dep]
-
-        if requirements_map:
-            while True:
-                to_clean = set()
-                for dep, required_by in requirements_map.items():
-                    if not required_by.difference(unnecessary):
-                        unnecessary.add(dep)
-                        to_clean.add(dep)
-
-                if to_clean:
-                    for dep in to_clean:
-                        del requirements_map[dep]
-                else:
-                    break
-
-            if requirements_map:  # if it reaches this points it is possible to exist mutual dependent packages
-                for dep, required_by in requirements_map.items():
-                    if not required_by.difference({*requirements_map.keys(), *unnecessary}):
-                        unnecessary.add(dep)
-
-    return unnecessary.difference(pkgs)
-
-
 def list_installed_names() -> Set[str]:
     output = run_cmd('pacman -Qq', print_error=False)
     return {name.strip() for name in output.split('\n') if name} if output else set()
