@@ -18,19 +18,20 @@ from requests import Response
 
 from bauh.api.abstract.context import ApplicationContext
 from bauh.api.abstract.controller import SoftwareManager, SearchResult, UpgradeRequirements, TransactionResult, \
-    SoftwareAction
+    SoftwareAction, SettingsView, SettingsController
 from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import SoftwarePackage, CustomSoftwareAction, PackageSuggestion, PackageUpdate, \
     PackageHistory, \
     SuggestionPriority, PackageStatus
 from bauh.api.abstract.view import MessageType, MultipleSelectComponent, InputOption, SingleSelectComponent, \
-    SelectViewType, TextInputComponent, FormComponent, FileChooserComponent, ViewComponent, PanelComponent
+    SelectViewType, TextInputComponent, FormComponent, FileChooserComponent, PanelComponent
 from bauh.api.paths import DESKTOP_ENTRIES_DIR
 from bauh.commons import resource
 from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import bold
-from bauh.commons.system import ProcessHandler, get_dir_size, get_human_size_str, SimpleProcess
+from bauh.commons.system import ProcessHandler, get_dir_size, SimpleProcess
+from bauh.commons.view_utils import get_human_size_str
 from bauh.gems.web import INSTALLED_PATH, nativefier, DESKTOP_ENTRY_PATH_PATTERN, URL_FIX_PATTERN, ENV_PATH, UA_CHROME, \
     SUGGESTIONS_CACHE_FILE, ROOT_DIR, TEMP_PATH, FIX_FILE_PATH, ELECTRON_CACHE_DIR, \
     get_icon_path, URL_PROPS_PATTERN
@@ -58,8 +59,10 @@ RE_PROTOCOL_STRIP = re.compile(r'[a-zA-Z]+://')
 RE_SEVERAL_SPACES = re.compile(r'\s+')
 RE_SYMBOLS_SPLIT = re.compile(r'[\-|_\s:.]')
 
+DEFAULT_LANGUAGE_HEADER = 'en-US, en'
 
-class WebApplicationManager(SoftwareManager):
+
+class WebApplicationManager(SoftwareManager, SettingsController):
 
     def __init__(self, context: ApplicationContext, suggestions_loader: Optional[SuggestionsLoader] = None):
         super(WebApplicationManager, self).__init__(context=context)
@@ -76,14 +79,30 @@ class WebApplicationManager(SoftwareManager):
         self.idxman = SearchIndexManager(logger=context.logger)
         self._custom_actions: Optional[Iterable[CustomSoftwareAction]] = None
 
-    def _get_lang_header(self) -> str:
+    def get_accept_language_header(self) -> str:
         try:
-            system_locale = locale.getdefaultlocale()
-            return system_locale[0] if system_locale else 'en_US'
-        except:
-            return 'en_US'
+            syslocale = locale.getdefaultlocale()
 
-    def clean_environment(self, root_password: str, watcher: ProcessWatcher) -> bool:
+            if syslocale:
+                locale_split = syslocale[0].split('_')
+
+                if len(locale_split) == 2:
+                    sys_lang = f'{locale_split[0]}-{locale_split[1]}, {locale_split[0]}'
+
+                    if DEFAULT_LANGUAGE_HEADER.split(',')[1].strip() not in sys_lang:
+                        return f'{sys_lang}, {DEFAULT_LANGUAGE_HEADER}'
+
+                    return sys_lang
+
+                else:
+                    return f'{syslocale[0]}, {DEFAULT_LANGUAGE_HEADER}'
+            else:
+                return DEFAULT_LANGUAGE_HEADER
+
+        except:
+            return DEFAULT_LANGUAGE_HEADER
+
+    def clean_environment(self, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         handler = ProcessHandler(watcher)
 
         success = True
@@ -220,7 +239,7 @@ class WebApplicationManager(SoftwareManager):
         super(WebApplicationManager, self).serialize_to_disk(pkg=pkg, icon_bytes=None, only_icon=False)
 
     def _request_url(self, url: str) -> Optional[Response]:
-        headers = {'Accept-language': self._get_lang_header(), 'User-Agent': UA_CHROME}
+        headers = {'Accept-language': self.get_accept_language_header(), 'User-Agent': UA_CHROME}
 
         try:
             return self.http_client.get(url, headers=headers, ignore_ssl=True, single_call=True, session=False, allow_redirects=True)
@@ -368,13 +387,13 @@ class WebApplicationManager(SoftwareManager):
 
         return res
 
-    def downgrade(self, pkg: SoftwarePackage, root_password: str, handler: ProcessWatcher) -> bool:
+    def downgrade(self, pkg: SoftwarePackage, root_password: Optional[str], handler: ProcessWatcher) -> bool:
         pass
 
-    def upgrade(self, requirements: UpgradeRequirements, root_password: str, watcher: ProcessWatcher) -> bool:
+    def upgrade(self, requirements: UpgradeRequirements, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         pass
 
-    def uninstall(self, pkg: WebApplication, root_password: str, watcher: ProcessWatcher, disk_loader: DiskCacheLoader) -> TransactionResult:
+    def uninstall(self, pkg: WebApplication, root_password: Optional[str], watcher: ProcessWatcher, disk_loader: DiskCacheLoader) -> TransactionResult:
         self.logger.info("Checking if {} installation directory {} exists".format(pkg.name, pkg.installation_dir))
 
         if not os.path.exists(pkg.installation_dir):
@@ -470,13 +489,17 @@ class WebApplicationManager(SoftwareManager):
     def _ask_install_options(self, app: WebApplication, watcher: ProcessWatcher, pre_validated: bool) -> Tuple[bool, List[str]]:
         watcher.change_substatus(self.i18n['web.install.substatus.options'])
 
-        inp_url = TextInputComponent(label=self.i18n['address'].capitalize() + ' (URL)', capitalize_label=False, value=app.url,
-                                     read_only=pre_validated, placeholder=f"({self.i18n['example.short']}: https://myapp123.com)")
-        inp_name = TextInputComponent(label=self.i18n['name'], value=app.name)
-        inp_desc = TextInputComponent(label=self.i18n['description'], value=app.description)
+        max_width = 350
+
+        inp_url = TextInputComponent(label=self.i18n['address'].capitalize() + ' (URL)', capitalize_label=False,
+                                     value=app.url, read_only=pre_validated, max_width=max_width,
+                                     placeholder=f"({self.i18n['example.short']}: https://myapp123.com)")
+        inp_name = TextInputComponent(label=self.i18n['name'], value=app.name, max_width=max_width)
+        inp_desc = TextInputComponent(label=self.i18n['description'], value=app.description, max_width=max_width)
 
         cat_ops = [InputOption(label=self.i18n['web.install.option.category.none'].capitalize(), value=0)]
-        cat_ops.extend([InputOption(label=self.i18n.get('category.{}'.format(c.lower()), c).capitalize(), value=c) for c in self.context.default_categories])
+        cat_ops.extend([InputOption(label=self.i18n.get('category.{}'.format(c.lower()), c).capitalize(), value=c)
+                        for c in self.context.default_categories])
 
         def_cat = cat_ops[0]
 
@@ -491,11 +514,16 @@ class WebApplicationManager(SoftwareManager):
                     def_cat = op
                     break
 
-        inp_cat = SingleSelectComponent(label=self.i18n['category'], type_=SelectViewType.COMBO, options=cat_ops, default_option=def_cat)
-        op_wv = InputOption(id_='widevine', label=self.i18n['web.install.option.widevine.label'] + ' (DRM)', value="--widevine", tooltip=self.i18n['web.install.option.widevine.tip'])
-        tray_op_off = InputOption(id_='tray_off', label=self.i18n['web.install.option.tray.off.label'], value=0, tooltip=self.i18n['web.install.option.tray.off.tip'])
-        tray_op_default = InputOption(id_='tray_def', label=self.i18n['web.install.option.tray.default.label'], value='--tray', tooltip=self.i18n['web.install.option.tray.default.tip'])
-        tray_op_min = InputOption(id_='tray_min', label=self.i18n['web.install.option.tray.min.label'], value='--tray=start-in-tray', tooltip=self.i18n['web.install.option.tray.min.tip'])
+        inp_cat = SingleSelectComponent(label=self.i18n['category'], type_=SelectViewType.COMBO, options=cat_ops,
+                                        default_option=def_cat, max_width=max_width)
+        op_wv = InputOption(id_='widevine', label=self.i18n['web.install.option.widevine.label'] + ' (DRM)',
+                            value="--widevine", tooltip=self.i18n['web.install.option.widevine.tip'])
+        tray_op_off = InputOption(id_='tray_off', label=self.i18n['web.install.option.tray.off.label'], value=0,
+                                  tooltip=self.i18n['web.install.option.tray.off.tip'])
+        tray_op_default = InputOption(id_='tray_def', label=self.i18n['web.install.option.tray.default.label'],
+                                      value='--tray', tooltip=self.i18n['web.install.option.tray.default.tip'])
+        tray_op_min = InputOption(id_='tray_min', label=self.i18n['web.install.option.tray.min.label'],
+                                  value='--tray=start-in-tray', tooltip=self.i18n['web.install.option.tray.min.tip'])
 
         tray_opts = [tray_op_off, tray_op_default, tray_op_min]
         def_tray_opt = None
@@ -506,10 +534,8 @@ class WebApplicationManager(SoftwareManager):
                     def_tray_opt = opt
                     break
 
-        inp_tray = SingleSelectComponent(type_=SelectViewType.COMBO,
-                                         options=tray_opts,
-                                         default_option=def_tray_opt,
-                                         label=self.i18n['web.install.option.tray.label'])
+        inp_tray = SingleSelectComponent(type_=SelectViewType.COMBO, options=tray_opts, default_option=def_tray_opt,
+                                         label=self.i18n['web.install.option.tray.label'], max_width=max_width)
 
         icon_op_ded = InputOption(id_='icon_ded', label=self.i18n['web.install.option.wicon.deducted.label'], value=0,
                                   tooltip=self.i18n['web.install.option.wicon.deducted.tip'].format('Nativefier'))
@@ -523,20 +549,32 @@ class WebApplicationManager(SoftwareManager):
         inp_icon = SingleSelectComponent(type_=SelectViewType.COMBO,
                                          options=[op for op in (icon_op_ded, icon_op_disp) if op],
                                          default_option=icon_op_disp if app.icon_url and app.save_icon else icon_op_ded,
-                                         label=self.i18n['web.install.option.wicon.label'])
+                                         label=self.i18n['web.install.option.wicon.label'],
+                                         max_width=max_width)
 
-        icon_chooser = FileChooserComponent(allowed_extensions={'png', 'svg', 'ico', 'jpg', 'jpeg'}, label=self.i18n['web.install.option.icon.label'])
+        icon_chooser = FileChooserComponent(allowed_extensions={'png', 'svg', 'ico', 'jpg', 'jpeg'},
+                                            label=self.i18n['web.install.option.icon.label'], max_width=max_width)
 
-        form_1 = FormComponent(components=[inp_url, inp_name, inp_desc, inp_cat, inp_icon, icon_chooser, inp_tray], label=self.i18n['web.install.options.basic'].capitalize())
+        form_1 = FormComponent(components=[inp_url, inp_name, inp_desc, inp_cat, inp_icon, icon_chooser, inp_tray],
+                               label=self.i18n['web.install.options.basic'].capitalize(), min_width=max_width)
 
-        op_single = InputOption(id_='single', label=self.i18n['web.install.option.single.label'], value="--single-instance", tooltip=self.i18n['web.install.option.single.tip'])
-        op_max = InputOption(id_='max', label=self.i18n['web.install.option.max.label'], value="--maximize", tooltip=self.i18n['web.install.option.max.tip'])
-        op_fs = InputOption(id_='fullscreen', label=self.i18n['web.install.option.fullscreen.label'], value="--full-screen", tooltip=self.i18n['web.install.option.fullscreen.tip'])
-        op_nframe = InputOption(id_='no_frame', label=self.i18n['web.install.option.noframe.label'], value="--hide-window-frame", tooltip=self.i18n['web.install.option.noframe.tip'])
-        op_allow_urls = InputOption(id_='allow_urls', label=self.i18n['web.install.option.allow_urls.label'], value='--internal-urls=.*', tooltip=self.i18n['web.install.option.allow_urls.tip'])
-        op_ncache = InputOption(id_='no_cache', label=self.i18n['web.install.option.nocache.label'], value="--clear-cache", tooltip=self.i18n['web.install.option.nocache.tip'])
-        op_insecure = InputOption(id_='insecure', label=self.i18n['web.install.option.insecure.label'], value="--insecure", tooltip=self.i18n['web.install.option.insecure.tip'])
-        op_igcert = InputOption(id_='ignore_certs', label=self.i18n['web.install.option.ignore_certificate.label'], value="--ignore-certificate", tooltip=self.i18n['web.install.option.ignore_certificate.tip'])
+        op_single = InputOption(id_='single', label=self.i18n['web.install.option.single.label'],
+                                value="--single-instance", tooltip=self.i18n['web.install.option.single.tip'])
+        op_max = InputOption(id_='max', label=self.i18n['web.install.option.max.label'], value="--maximize",
+                             tooltip=self.i18n['web.install.option.max.tip'])
+        op_fs = InputOption(id_='fullscreen', label=self.i18n['web.install.option.fullscreen.label'],
+                            value="--full-screen", tooltip=self.i18n['web.install.option.fullscreen.tip'])
+        op_nframe = InputOption(id_='no_frame', label=self.i18n['web.install.option.noframe.label'],
+                                value="--hide-window-frame", tooltip=self.i18n['web.install.option.noframe.tip'])
+        op_allow_urls = InputOption(id_='allow_urls', label=self.i18n['web.install.option.allow_urls.label'],
+                                    value='--internal-urls=.*', tooltip=self.i18n['web.install.option.allow_urls.tip'])
+        op_ncache = InputOption(id_='no_cache', label=self.i18n['web.install.option.nocache.label'],
+                                value="--clear-cache", tooltip=self.i18n['web.install.option.nocache.tip'])
+        op_insecure = InputOption(id_='insecure', label=self.i18n['web.install.option.insecure.label'],
+                                  value="--insecure", tooltip=self.i18n['web.install.option.insecure.tip'])
+        op_igcert = InputOption(id_='ignore_certs', label=self.i18n['web.install.option.ignore_certificate.label'],
+                                value="--ignore-certificate",
+                                tooltip=self.i18n['web.install.option.ignore_certificate.tip'])
 
         adv_opts = [op_single, op_allow_urls, op_wv, op_max, op_fs, op_nframe, op_ncache, op_insecure, op_igcert]
         def_adv_opts = {op_single, op_allow_urls}
@@ -546,23 +584,29 @@ class WebApplicationManager(SoftwareManager):
                 if opt.id in app.preset_options:
                     def_adv_opts.add(opt)
 
-        check_options = MultipleSelectComponent(options=adv_opts, default_options=def_adv_opts, label=self.i18n['web.install.options.advanced'].capitalize())
+        check_options = MultipleSelectComponent(options=adv_opts, default_options=def_adv_opts, max_width=480,
+                                                min_width=max_width,
+                                                label=self.i18n['web.install.options.advanced'].capitalize(),
+                                                opt_max_width=450)
 
         install_ = watcher.request_confirmation(title=self.i18n['web.install.options_dialog.title'],
                                                 body=None,
                                                 components=[form_1, check_options],
                                                 confirmation_label=self.i18n['continue'].capitalize(),
-                                                deny_label=self.i18n['cancel'].capitalize())
+                                                deny_label=self.i18n['cancel'].capitalize(),
+                                                min_width=max_width + 100)
 
         if install_:
             if not pre_validated:
                 typed_url = inp_url.get_value().strip()
 
                 if not typed_url or not self._request_url(typed_url):
+                    url_ = inp_url.get_value()
+                    msg = self.i18n['web.custom_action.install_app.invalid_url'].format(URL='(URL)',
+                                                                                        url=bold(f'"{url_}"'))
                     watcher.show_message(title=self.i18n['error'].capitalize(),
                                          type_=MessageType.ERROR,
-                                         body=self.i18n['web.custom_action.install_app.invalid_url'].format(URL='(URL)',
-                                                                                                            url=bold(f'"{inp_url.get_value()}"')))
+                                         body=msg)
                     return False, []
                 else:
                     app.url = typed_url
@@ -842,7 +886,7 @@ class WebApplicationManager(SoftwareManager):
 
         return TransactionResult(success=True, installed=[pkg], removed=[])
 
-    def install(self, pkg: WebApplication, root_password: str, disk_loader: DiskCacheLoader, watcher: ProcessWatcher) -> TransactionResult:
+    def install(self, pkg: WebApplication, root_password: Optional[str], disk_loader: DiskCacheLoader, watcher: ProcessWatcher) -> TransactionResult:
         continue_install, install_options = self._ask_install_options(pkg, watcher, pre_validated=True)
 
         if not continue_install:
@@ -850,7 +894,7 @@ class WebApplicationManager(SoftwareManager):
 
         return self._install(pkg, install_options, watcher)
 
-    def install_app(self, root_password: str, watcher: ProcessWatcher) -> bool:
+    def install_app(self, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         pkg = WebApplication()
         continue_install, install_options = self._ask_install_options(pkg, watcher, pre_validated=False)
 
@@ -908,7 +952,7 @@ class WebApplicationManager(SoftwareManager):
     def _assign_suggestions(self, suggestions: dict):
         self.suggestions = suggestions
 
-    def prepare(self, task_manager: TaskManager, root_password: str, internet_available: bool):
+    def prepare(self, task_manager: TaskManager, root_password: Optional[str], internet_available: bool):
         create_config = CreateConfigFile(taskman=task_manager, configman=self.configman, i18n=self.i18n,
                                          task_icon_path=get_icon_path(), logger=self.logger)
         create_config.start()
@@ -1004,7 +1048,7 @@ class WebApplicationManager(SoftwareManager):
     def _fill_config_async(self, output: dict):
         output.update(self.configman.get_config())
 
-    def list_suggestions(self, limit: int, filter_installed: bool) -> List[PackageSuggestion]:
+    def list_suggestions(self, limit: int, filter_installed: bool) -> Optional[List[PackageSuggestion]]:
         web_config = {}
 
         thread_config = Thread(target=self._fill_config_async, args=(web_config,))
@@ -1063,7 +1107,7 @@ class WebApplicationManager(SoftwareManager):
 
             return res
 
-    def execute_custom_action(self, action: CustomSoftwareAction, pkg: SoftwarePackage, root_password: str, watcher: ProcessWatcher) -> bool:
+    def execute_custom_action(self, action: CustomSoftwareAction, pkg: SoftwarePackage, root_password: Optional[str], watcher: ProcessWatcher) -> bool:
         pass
 
     def is_default_enabled(self) -> bool:
@@ -1072,9 +1116,6 @@ class WebApplicationManager(SoftwareManager):
     def launch(self, pkg: WebApplication):
         subprocess.Popen(args=[pkg.get_command()], shell=True, env={**os.environ})
 
-    def get_screenshots(self, pkg: SoftwarePackage) -> List[str]:
-        pass
-
     def clear_data(self, logs: bool = True):
         if os.path.exists(ENV_PATH):
             if logs:
@@ -1089,15 +1130,14 @@ class WebApplicationManager(SoftwareManager):
                     print('{}[bauh][web] An exception has happened when deleting {}{}'.format(Fore.RED, ENV_PATH, Fore.RESET))
                     traceback.print_exc()
 
-    def get_settings(self, screen_width: int, screen_height: int) -> Optional[ViewComponent]:
+    def get_settings(self) -> Optional[Generator[SettingsView, None, None]]:
         web_config = self.configman.get_config()
-        max_width = floor(screen_width * 0.15)
 
         input_electron = TextInputComponent(label=self.i18n['web.settings.electron.version.label'],
                                             value=web_config['environment']['electron']['version'],
                                             tooltip=self.i18n['web.settings.electron.version.tooltip'],
                                             placeholder='{}: 7.1.0'.format(self.i18n['example.short']),
-                                            max_width=max_width,
+                                            max_width=150,
                                             id_='electron_branch')
 
         native_opts = [
@@ -1110,7 +1150,7 @@ class WebApplicationManager(SoftwareManager):
                                                   default_option=[o for o in native_opts if o.value == web_config['environment']['system']][0],
                                                   type_=SelectViewType.COMBO,
                                                   tooltip=self.i18n['web.settings.nativefier.tip'],
-                                                  max_width=max_width,
+                                                  max_width=150,
                                                   id_='nativefier')
 
         env_settings_exp = TextInputComponent(label=self.i18n['web.settings.cache_exp'],
@@ -1118,7 +1158,7 @@ class WebApplicationManager(SoftwareManager):
                                               capitalize_label=False,
                                               value=int(web_config['environment']['cache_exp']) if isinstance(web_config['environment']['cache_exp'], int) else '',
                                               only_int=True,
-                                              max_width=max_width,
+                                              max_width=60,
                                               id_='web_cache_exp')
 
         sugs_exp = TextInputComponent(label=self.i18n['web.settings.suggestions.cache_exp'],
@@ -1127,35 +1167,36 @@ class WebApplicationManager(SoftwareManager):
                                       value=int(web_config['suggestions']['cache_exp']) if isinstance(
                                           web_config['suggestions']['cache_exp'], int) else '',
                                       only_int=True,
-                                      max_width=max_width,
+                                      max_width=60,
                                       id_='web_sugs_exp')
 
         form_env = FormComponent(label=self.i18n['web.settings.nativefier.env'].capitalize(),
-                                 components=[input_electron, select_nativefier, env_settings_exp, sugs_exp])
+                                 components=[select_nativefier, input_electron, env_settings_exp, sugs_exp])
 
-        return PanelComponent([form_env])
+        yield SettingsView(self, PanelComponent([form_env]))
 
     def save_settings(self, component: PanelComponent) -> Tuple[bool, Optional[List[str]]]:
-        web_config = self.configman.get_config()
+        config_ = self.configman.get_config()
 
-        form_env = component.components[0]
+        form = component.get_component_by_idx(0, FormComponent)
 
-        web_config['environment']['electron']['version'] = str(form_env.get_component('electron_branch').get_value()).strip()
+        electron_version = str(form.get_component('electron_branch', TextInputComponent).get_value()).strip()
+        config_['environment']['electron']['version'] = electron_version
 
-        if len(web_config['environment']['electron']['version']) == 0:
-            web_config['environment']['electron']['version'] = None
+        if len(config_['environment']['electron']['version']) == 0:
+            config_['environment']['electron']['version'] = None
 
-        system_nativefier = form_env.get_component('nativefier').get_selected()
+        system_nativefier = form.get_component('nativefier', SingleSelectComponent).get_selected()
 
         if system_nativefier and not nativefier.is_available():
             return False, [self.i18n['web.settings.env.nativefier.system.not_installed'].format('Nativefier')]
 
-        web_config['environment']['system'] = system_nativefier
-        web_config['environment']['cache_exp'] = form_env.get_component('web_cache_exp').get_int_value()
-        web_config['suggestions']['cache_exp'] = form_env.get_component('web_sugs_exp').get_int_value()
+        config_['environment']['system'] = system_nativefier
+        config_['environment']['cache_exp'] = form.get_component('web_cache_exp', TextInputComponent).get_int_value()
+        config_['suggestions']['cache_exp'] = form.get_component('web_sugs_exp', TextInputComponent).get_int_value()
 
         try:
-            self.configman.save_config(web_config)
+            self.configman.save_config(config_)
             return True, None
         except:
             return False, [traceback.format_exc()]
@@ -1165,6 +1206,7 @@ class WebApplicationManager(SoftwareManager):
             self._custom_actions = (
                 CustomSoftwareAction(i18n_label_key='web.custom_action.install_app',
                                      i18n_status_key='web.custom_action.install_app.status',
+                                     i18n_description_key='web.custom_action.install_app.desc',
                                      manager=self,
                                      manager_method='install_app',
                                      icon_path=resource.get_path('img/web.svg', ROOT_DIR),
@@ -1173,6 +1215,7 @@ class WebApplicationManager(SoftwareManager):
                                      requires_confirmation=False),
                 CustomSoftwareAction(i18n_label_key='web.custom_action.clean_env',
                                      i18n_status_key='web.custom_action.clean_env.status',
+                                     i18n_description_key='web.custom_action.clean_env.desc',
                                      manager=self,
                                      manager_method='clean_environment',
                                      icon_path=resource.get_path('img/web.svg', ROOT_DIR),
