@@ -521,10 +521,13 @@ class MultipleSelectQt(QGroupBox):
         self._layout = QGridLayout()
         self.setLayout(self._layout)
 
-        if model.max_width > 0:
+        if model.min_width and model.min_width > 0:
+            self.setMinimumWidth(int(model.min_width))
+
+        if model.max_width and model.max_width > 0:
             self.setMaximumWidth(int(model.max_width))
 
-        if model.max_height > 0:
+        if model.max_height and model.max_height > 0:
             self.setMaximumHeight(int(model.max_height))
 
         if model.label:
@@ -544,9 +547,13 @@ class MultipleSelectQt(QGroupBox):
                 comp.setChecked(True)
 
             widget = QWidget()
+            widget.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
             widget.setLayout(QHBoxLayout())
             widget.layout().addWidget(comp)
 
+            if model.opt_max_width and model.opt_max_width > 0:
+                widget.setMinimumWidth(int(model.opt_max_width))
+
             if op.tooltip:
                 help_icon = QLabel()
 
@@ -579,10 +586,13 @@ class FormMultipleSelectQt(QWidget):
         self.model = model
         self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
 
-        if model.max_width > 0:
+        if model.min_width and model.min_width > 0:
+            self.setMinimumWidth(int(model.min_width))
+
+        if model.max_width and model.max_width > 0:
             self.setMaximumWidth(int(model.max_width))
 
-        if model.max_height > 0:
+        if model.max_height and model.max_height > 0:
             self.setMaximumHeight(int(model.max_height))
 
         self._layout = QGridLayout()
@@ -604,9 +614,13 @@ class FormMultipleSelectQt(QWidget):
                 comp.setChecked(True)
 
             widget = QWidget()
+            widget.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
             widget.setLayout(QHBoxLayout())
             widget.layout().addWidget(comp)
 
+            if model.opt_max_width and model.opt_max_width > 0:
+                widget.setMinimumWidth(int(model.opt_max_width))
+
             if op.tooltip:
                 help_icon = QLabel()
 
@@ -707,6 +721,9 @@ class FormQt(QGroupBox):
         self.i18n = i18n
         self.setLayout(QFormLayout())
 
+        if model.min_width and model.min_width > 0:
+            self.setMinimumWidth(model.min_width)
+
         if model.spaces:
             self.layout().addRow(QLabel(), QLabel())
 
@@ -760,6 +777,9 @@ class FormQt(QGroupBox):
         label_comp = QLabel()
         label.layout().addWidget(label_comp)
 
+        if hasattr(comp, 'min_width') and comp.min_width is not None and comp.min_width > 0:
+            label_comp.setMinimumWidth(comp.min_width)
+
         if hasattr(comp, 'size') and comp.size is not None:
             label_comp.setStyleSheet("QLabel { font-size: " + str(comp.size) + "px }")
 
@@ -853,6 +873,7 @@ class FormQt(QGroupBox):
     def _wrap(self, comp: QWidget, model: ViewComponent) -> QWidget:
         field_container = QWidget()
         field_container.setLayout(QHBoxLayout())
+        field_container.layout().setContentsMargins(0, 0, 0, 0)
 
         if model.max_width > 0:
             field_container.setMaximumWidth(int(model.max_width))
@@ -938,7 +959,7 @@ class TabGroupQt(QTabWidget):
             scroll = QScrollArea()
             scroll.setFrameShape(QFrame.NoFrame)
             scroll.setWidgetResizable(True)
-            scroll.setWidget(to_widget(c.content, i18n))
+            scroll.setWidget(to_widget(c.get_content(), i18n))
             self.addTab(scroll, icon, c.label)
 
         self.tabBar().setCursor(QCursor(Qt.PointingHandCursor))
@@ -984,6 +1005,12 @@ def to_widget(comp: ViewComponent, i18n: I18n, parent: QWidget = None) -> QWidge
     elif isinstance(comp, TextComponent):
         label = QLabel(comp.value)
 
+        if comp.min_width is not None and comp.min_width > 0:
+            label.setMinimumWidth(comp.min_width)
+
+        if comp.max_width is not None and comp.max_width > 0:
+            label.setMinimumWidth(comp.max_width)
+
         if comp.size is not None:
             label.setStyleSheet("QLabel { font-size: " + str(comp.size) + "px }")
 
